{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE DuplicateRecordFields #-}
{-# LANGUAGE NamedFieldPuns #-}
{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE RecordWildCards #-}
{-# LANGUAGE StrictData #-}
{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE NoImplicitPrelude #-}
{-# OPTIONS_GHC -fno-warn-unused-binds #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-unused-matches #-}

-- Derived from AWS service descriptions, licensed under Apache 2.0.

-- |
-- Module      : Amazonka.IAM.TagOpenIDConnectProvider
-- Copyright   : (c) 2013-2023 Brendan Hay
-- License     : Mozilla Public License, v. 2.0.
-- Maintainer  : Brendan Hay
-- Stability   : auto-generated
-- Portability : non-portable (GHC extensions)
--
-- Adds one or more tags to an OpenID Connect (OIDC)-compatible identity
-- provider. For more information about these providers, see
-- <https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_providers_oidc.html About web identity federation>.
-- If a tag with the same key name already exists, then that tag is
-- overwritten with the new value.
--
-- A tag consists of a key name and an associated value. By assigning tags
-- to your resources, you can do the following:
--
-- -   __Administrative grouping and discovery__ - Attach tags to resources
--     to aid in organization and search. For example, you could search for
--     all resources with the key name /Project/ and the value
--     /MyImportantProject/. Or search for all resources with the key name
--     /Cost Center/ and the value /41200/.
--
-- -   __Access control__ - Include tags in IAM user-based and
--     resource-based policies. You can use tags to restrict access to only
--     an OIDC provider that has a specified tag attached. For examples of
--     policies that show how to use tags to control access, see
--     <https://docs.aws.amazon.com/IAM/latest/UserGuide/access_tags.html Control access using IAM tags>
--     in the /IAM User Guide/.
--
-- -   If any one of the tags is invalid or if you exceed the allowed
--     maximum number of tags, then the entire request fails and the
--     resource is not created. For more information about tagging, see
--     <https://docs.aws.amazon.com/IAM/latest/UserGuide/id_tags.html Tagging IAM resources>
--     in the /IAM User Guide/.
--
-- -   Amazon Web Services always interprets the tag @Value@ as a single
--     string. If you need to store an array, you can store comma-separated
--     values in the string. However, you must interpret the value in your
--     code.
module Amazonka.IAM.TagOpenIDConnectProvider
  ( -- * Creating a Request
    TagOpenIDConnectProvider (..),
    newTagOpenIDConnectProvider,

    -- * Request Lenses
    tagOpenIDConnectProvider_openIDConnectProviderArn,
    tagOpenIDConnectProvider_tags,

    -- * Destructuring the Response
    TagOpenIDConnectProviderResponse (..),
    newTagOpenIDConnectProviderResponse,
  )
where

import qualified Amazonka.Core as Core
import qualified Amazonka.Core.Lens.Internal as Lens
import qualified Amazonka.Data as Data
import Amazonka.IAM.Types
import qualified Amazonka.Prelude as Prelude
import qualified Amazonka.Request as Request
import qualified Amazonka.Response as Response

-- | /See:/ 'newTagOpenIDConnectProvider' smart constructor.
data TagOpenIDConnectProvider = TagOpenIDConnectProvider'
  { -- | The ARN of the OIDC identity provider in IAM to which you want to add
    -- tags.
    --
    -- This parameter allows (through its
    -- <http://wikipedia.org/wiki/regex regex pattern>) a string of characters
    -- consisting of upper and lowercase alphanumeric characters with no
    -- spaces. You can also include any of the following characters: _+=,.\@-
    openIDConnectProviderArn :: Prelude.Text,
    -- | The list of tags that you want to attach to the OIDC identity provider
    -- in IAM. Each tag consists of a key name and an associated value.
    tags :: [Tag]
  }
  deriving (Prelude.Eq, Prelude.Read, Prelude.Show, Prelude.Generic)

-- |
-- Create a value of 'TagOpenIDConnectProvider' with all optional fields omitted.
--
-- Use <https://hackage.haskell.org/package/generic-lens generic-lens> or <https://hackage.haskell.org/package/optics optics> to modify other optional fields.
--
-- The following record fields are available, with the corresponding lenses provided
-- for backwards compatibility:
--
-- 'openIDConnectProviderArn', 'tagOpenIDConnectProvider_openIDConnectProviderArn' - The ARN of the OIDC identity provider in IAM to which you want to add
-- tags.
--
-- This parameter allows (through its
-- <http://wikipedia.org/wiki/regex regex pattern>) a string of characters
-- consisting of upper and lowercase alphanumeric characters with no
-- spaces. You can also include any of the following characters: _+=,.\@-
--
-- 'tags', 'tagOpenIDConnectProvider_tags' - The list of tags that you want to attach to the OIDC identity provider
-- in IAM. Each tag consists of a key name and an associated value.
newTagOpenIDConnectProvider ::
  -- | 'openIDConnectProviderArn'
  Prelude.Text ->
  TagOpenIDConnectProvider
newTagOpenIDConnectProvider
  pOpenIDConnectProviderArn_ =
    TagOpenIDConnectProvider'
      { openIDConnectProviderArn =
          pOpenIDConnectProviderArn_,
        tags = Prelude.mempty
      }

-- | The ARN of the OIDC identity provider in IAM to which you want to add
-- tags.
--
-- This parameter allows (through its
-- <http://wikipedia.org/wiki/regex regex pattern>) a string of characters
-- consisting of upper and lowercase alphanumeric characters with no
-- spaces. You can also include any of the following characters: _+=,.\@-
tagOpenIDConnectProvider_openIDConnectProviderArn :: Lens.Lens' TagOpenIDConnectProvider Prelude.Text
tagOpenIDConnectProvider_openIDConnectProviderArn = Lens.lens (\TagOpenIDConnectProvider' {openIDConnectProviderArn} -> openIDConnectProviderArn) (\s@TagOpenIDConnectProvider' {} a -> s {openIDConnectProviderArn = a} :: TagOpenIDConnectProvider)

-- | The list of tags that you want to attach to the OIDC identity provider
-- in IAM. Each tag consists of a key name and an associated value.
tagOpenIDConnectProvider_tags :: Lens.Lens' TagOpenIDConnectProvider [Tag]
tagOpenIDConnectProvider_tags = Lens.lens (\TagOpenIDConnectProvider' {tags} -> tags) (\s@TagOpenIDConnectProvider' {} a -> s {tags = a} :: TagOpenIDConnectProvider) Prelude.. Lens.coerced

instance Core.AWSRequest TagOpenIDConnectProvider where
  type
    AWSResponse TagOpenIDConnectProvider =
      TagOpenIDConnectProviderResponse
  request overrides =
    Request.postQuery (overrides defaultService)
  response =
    Response.receiveNull
      TagOpenIDConnectProviderResponse'

instance Prelude.Hashable TagOpenIDConnectProvider where
  hashWithSalt _salt TagOpenIDConnectProvider' {..} =
    _salt
      `Prelude.hashWithSalt` openIDConnectProviderArn
      `Prelude.hashWithSalt` tags

instance Prelude.NFData TagOpenIDConnectProvider where
  rnf TagOpenIDConnectProvider' {..} =
    Prelude.rnf openIDConnectProviderArn
      `Prelude.seq` Prelude.rnf tags

instance Data.ToHeaders TagOpenIDConnectProvider where
  toHeaders = Prelude.const Prelude.mempty

instance Data.ToPath TagOpenIDConnectProvider where
  toPath = Prelude.const "/"

instance Data.ToQuery TagOpenIDConnectProvider where
  toQuery TagOpenIDConnectProvider' {..} =
    Prelude.mconcat
      [ "Action"
          Data.=: ("TagOpenIDConnectProvider" :: Prelude.ByteString),
        "Version"
          Data.=: ("2010-05-08" :: Prelude.ByteString),
        "OpenIDConnectProviderArn"
          Data.=: openIDConnectProviderArn,
        "Tags" Data.=: Data.toQueryList "member" tags
      ]

-- | /See:/ 'newTagOpenIDConnectProviderResponse' smart constructor.
data TagOpenIDConnectProviderResponse = TagOpenIDConnectProviderResponse'
  {
  }
  deriving (Prelude.Eq, Prelude.Read, Prelude.Show, Prelude.Generic)

-- |
-- Create a value of 'TagOpenIDConnectProviderResponse' with all optional fields omitted.
--
-- Use <https://hackage.haskell.org/package/generic-lens generic-lens> or <https://hackage.haskell.org/package/optics optics> to modify other optional fields.
newTagOpenIDConnectProviderResponse ::
  TagOpenIDConnectProviderResponse
newTagOpenIDConnectProviderResponse =
  TagOpenIDConnectProviderResponse'

instance
  Prelude.NFData
    TagOpenIDConnectProviderResponse
  where
  rnf _ = ()
